/*
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2011 KYOCERA Corporation  
*/

#include <linux/module.h>
#include <linux/delay.h>

#include <mach/hs_io_ctl_a.h>
#include <mach/vreg.h>
#include <mach/kc_bcm_signal_control.h>

#define BCM_DEBUG 1

static int wifi_status;
static int bt_status;

static struct vreg *vreg_ruim;

void kc_bcm_signal_init(void)
{
	HS_A_REG_ON_LO();
	HS_A_WL_RST_N_LO();

	HS_A_REG_ON_LO();
	HS_A_WL_RST_N_LO();

	wifi_status = 0;
	bt_status = 0;
}

void bcm_wlan_power_on(int param)
{
#if BCM_DEBUG
  printk(KERN_INFO "bcm_wlan_power_on() CALLED, param = %d\n", param);
#endif

  HS_A_WL_WAKE_N_LO();

  if (param == 1)
  {
#if BCM_DEBUG
  	printk(KERN_INFO "Turn on wlan power (RUIM, WL_REG_ON, WL_RST_N) \n");
#endif

	if (wifi_status == 0)
	{
		if (bt_status == 0)
		{
#if BCM_DEBUG
			printk(KERN_INFO "Bluetooth OFF, Turn On Power\n");
#endif
			hs_vreg_ruim_enable();

			msleep(1);
			
			HS_A_REG_ON_HI();

		} else {
#if BCM_DEBUG
			printk(KERN_INFO "Bluetooth ON, No need to Turn On Power\n");
#endif
		}

		msleep(100);

		HS_A_WL_RST_N_HI();

		msleep(100);

		wifi_status = 1;
	}

   } else {
#if BCM_DEBUG
  	printk(KERN_INFO "Turn on WL_RST_N (Wake from sleep ? Not deeply consider yet)\n");
#endif
	HS_A_WL_RST_N_HI();

	msleep(100);	
   }
}
EXPORT_SYMBOL(bcm_wlan_power_on);

void bcm_wlan_power_off(int param)
{
#if BCM_DEBUG
  printk(KERN_INFO "bcm_wlan_power_off() CALLED, param = %d\n", param);
#endif

  if (param == 1)
  {
#if BCM_DEBUG
	printk(KERN_INFO "Turn off wlan power (RUIM, WL_REG_ON, WL_RST_N)\n");
#endif

	if (wifi_status == 1)
	{
		msleep(100);

		HS_A_WL_RST_N_LO();

		msleep(100);

		if (bt_status == 0)
		{
#if BCM_DEBUG
			printk(KERN_INFO "Bluetooth OFF, Turn Off Power\n");
#endif

			HS_A_REG_ON_LO();

			msleep(100);

		        hs_vreg_ruim_disable();
		} else {
#if BCM_DEBUG
			printk(KERN_INFO "Bluetooth ON, Cannot Turn Off Power\n");
#endif
		}

		wifi_status = 0;
	}

   } else {
#if BCM_DEBUG
  	printk(KERN_INFO "Turn off WL_RST_N (Change to sleep ? Not deeply consider yet)\n");
#endif

	HS_A_WL_RST_N_LO();

	msleep(100);	
   }
}
EXPORT_SYMBOL(bcm_wlan_power_off);

void kc_bcm_bt_signal_ctl(int on)
{
   if(on)
   { 

#if BCM_DEBUG
	printk(KERN_INFO "kc_bcm_bt_signal_ctl() CALLED, ON\n");
#endif

	if (bt_status == 0)
	{
		if (wifi_status == 0)
		{
#if BCM_DEBUG
			printk(KERN_INFO "WiFi OFF, Turn On Power\n");
#endif
			hs_vreg_ruim_enable();

			msleep(1);

			HS_A_REG_ON_HI();

		} else {
#if BCM_DEBUG
			printk(KERN_INFO "WiFi ON, No need to Turn On Power\n");
#endif
		}

			msleep(100);

			HS_A_BT_RST_HI();

			msleep(1000);

			bt_status = 1;
	}

   } else {

#if BCM_DEBUG
	printk(KERN_INFO "kc_bcm_bt_signal_ctl() CALLED, OFF\n");
#endif

	if (bt_status == 1)
	{
		HS_A_BT_RST_LO();

		if (wifi_status == 0)
		{
#if BCM_DEBUG
			printk(KERN_INFO "WiFi OFF, Turn Off Power\n");
#endif
			HS_A_REG_ON_LO();

		        hs_vreg_ruim_disable();

		} else {
#if BCM_DEBUG
			printk(KERN_INFO "WiFi ON, Cannot Turn Off Power\n");
#endif
		}

		bt_status = 0;
	}
   }
}
EXPORT_SYMBOL(kc_bcm_bt_signal_ctl);

void bcm_bt_power_off(void)
{
#if BCM_DEBUG
	printk(KERN_INFO "bcm_bt_power_off() CALLED\n");
#endif

	if (bt_status == 1)
	{
		HS_A_BT_RST_LO();

		if (wifi_status == 0)
		{
#if BCM_DEBUG
			printk(KERN_INFO "WiFi OFF, Turn Off Power\n");
#endif
			HS_A_REG_ON_LO();

		        hs_vreg_ruim_disable();

		} else {
#if BCM_DEBUG
			printk(KERN_INFO "WiFi ON, Cannot Turn Off Power\n");
#endif
		}

		bt_status = 0;
	}
}
EXPORT_SYMBOL(bcm_bt_power_off);

void bcm_bt_power_on(void)
{
#if BCM_DEBUG
	printk(KERN_INFO "bcm_bt_power_on() CALLED\n");
#endif

	if (bt_status == 0)
	{
		if (wifi_status == 0)
		{
#if BCM_DEBUG
			printk(KERN_INFO "WiFi OFF, Turn On Power\n");
#endif
			hs_vreg_ruim_enable();

			msleep(1);

			HS_A_REG_ON_HI();

		} else {
#if BCM_DEBUG
			printk(KERN_INFO "WiFi ON, No need to Turn On Power\n");
#endif
		}

			msleep(100);

			HS_A_BT_RST_HI();

			msleep(1000);

			bt_status = 1;
	}
}
EXPORT_SYMBOL(bcm_bt_power_on);

void hs_vreg_ruim_enable( void )
{
  int rc = 0;
  
  vreg_ruim = vreg_get(NULL, "ruim");

  rc = vreg_set_level(vreg_ruim, 2850);
  if (!rc)
  {
    rc = vreg_enable(vreg_ruim);
  }

#if BCM_DEBUG
  printk(KERN_INFO "hs_vreg_ruim_enable() results = %d\n", rc);
#endif
}
EXPORT_SYMBOL(hs_vreg_ruim_enable);

void hs_vreg_ruim_disable( void )
{
  int rc = 0;

  vreg_ruim = vreg_get(NULL, "ruim");

  rc = vreg_disable(vreg_ruim);

#if BCM_DEBUG
  printk(KERN_INFO "hs_vreg_ruim_disable() results = %d\n", rc);
#endif
}
EXPORT_SYMBOL(hs_vreg_ruim_disable);
