/*
 * leds-msm-pmic.c - MSM PMIC LEDs driver.
 *
 * Copyright (c) 2009, Code Aurora Forum. All rights reserved.
 *
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2013 KYOCERA Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */








#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/leds.h>

#include <linux/fs.h>
#include <linux/syscalls.h>
#include <linux/unistd.h>
#include <linux/fcntl.h>


#include <mach/pmic.h>

#include <mach/../../proc_comm.h>


#define MAX_KEYPAD_BL_LEVEL	16


#define PROC_COMM_SUB_CMD_LED_CONTROL_1      2
#define PROC_COMM_SUB_CMD_LED_CONTROL_2      3


  
  
  
  
  
  
  
#define LED_KEYPAD_ON      3


struct light_state_t {
	unsigned int color;
	int flashMode;
	int flashOnMS;
	int flashOffMS;
	int brightnessMode;
};

static struct timer_list        timer_led;
static struct workqueue_struct *led_work;
static struct work_struct       work_led;
static struct workqueue_struct *ledstate_work;
static struct work_struct       work_ledstate;
static int led_value;
static int fs_state;



static int key_bl_val    = -1;
static int button_bl_val = -1;


static void msm_keypad_bl_led_set(struct led_classdev *led_cdev,
	enum led_brightness value)
{
	int ret;

	if (value == key_bl_val)  
		return;



	if (value == LED_OFF) {
		ret = pmic_set_led_intensity(LED_KEYPAD, LED_OFF);
	} else {
		ret = pmic_set_led_intensity(LED_KEYPAD, LED_KEYPAD_ON);
	}

	if (ret)
		dev_err(led_cdev->dev, "can't set keypad backlight\n");
}

static struct led_classdev msm_kp_bl_led = {
	.name			= "keyboard-backlight",
	.brightness_set		= msm_keypad_bl_led_set,
	.brightness		= LED_OFF,
};


static void msm_button_bl_led_set(struct led_classdev *led_cdev,
	enum led_brightness value)
{
	if (value == button_bl_val)  
		return;


	if(value){
		pmic_secure_mpp_config_i_sink(PM_MPP_20,PM_MPP__I_SINK__LEVEL_10mA,PM_MPP__I_SINK__SWITCH_ENA);
	}
	else{
		pmic_secure_mpp_config_i_sink(PM_MPP_20,PM_MPP__I_SINK__LEVEL_10mA,PM_MPP__I_SINK__SWITCH_DIS);
	}


}

static struct led_classdev msm_button_bl_led = {
	.name			= "button-backlight",
	.brightness_set		= msm_button_bl_led_set,
	.brightness		= LED_OFF,
};



static void msm_green_led_set(struct led_classdev *led_cdev,enum led_brightness value)
{
	if(value){
		pmic_secure_mpp_config_i_sink(PM_MPP_17,PM_MPP__I_SINK__LEVEL_5mA,PM_MPP__I_SINK__SWITCH_ENA);
	}
	else{
		pmic_secure_mpp_config_i_sink(PM_MPP_17,PM_MPP__I_SINK__LEVEL_5mA,PM_MPP__I_SINK__SWITCH_DIS);
	}
}
static void msm_red_led_set(struct led_classdev *led_cdev,enum led_brightness value)
{
	if(value){
		pmic_secure_mpp_config_i_sink(PM_MPP_18,PM_MPP__I_SINK__LEVEL_5mA,PM_MPP__I_SINK__SWITCH_ENA);
	}
	else{
		pmic_secure_mpp_config_i_sink(PM_MPP_18,PM_MPP__I_SINK__LEVEL_5mA,PM_MPP__I_SINK__SWITCH_DIS);
	}
}

static struct led_classdev msm_red_led = {
	.name			= "red",
	.brightness_set		= msm_red_led_set,
	.brightness		= LED_OFF,
};
static struct led_classdev msm_green_led = {
	.name			= "green",
	.brightness_set		= msm_green_led_set,
	.brightness		= LED_OFF,
};



static void msm_blue_led_set(struct led_classdev *led_cdev, enum led_brightness value)
{
	printk("led BLUE brightness %d\n",value);
	
}

static void msm_state_led_set(struct led_classdev *led_cdev, enum led_brightness value)
{
	printk("led state brightness %d\n",value);

	if(value > 0 && 6 > value){
		led_value = value;

		if(fs_state)queue_work(ledstate_work, &work_ledstate);
	}
}

static void ledstate_work_0(struct work_struct *work)
{

	struct file *filp;
	struct light_state_t  buf;
	int nr_read;
	int data1,data2;
	int color;

	filp = filp_open("/dev/ledstate", O_RDONLY, 0);
	if (!filp) {
		return;
	}

	
	nr_read = filp->f_op->read(filp, (void*)&buf, sizeof(struct light_state_t), &filp->f_pos);
	printk("********* led state start\n");
	printk("color : 0x%x\n", buf.color);
	printk("flashMode : %d\n", buf.flashMode);
	printk("flashOnMS : %d\n", buf.flashOnMS);
	printk("flashOffMS : %d\n", buf.flashOffMS);
	printk("brightnessMode : %d\n", buf.brightnessMode);
	printk("********* led state end\n");

	filp_close(filp, NULL);

	
	data1 = PROC_COMM_SUB_CMD_LED_CONTROL_1;
	color =  (buf.color&0x000000ff)?1:0;
	color += (buf.color&0x0000ff00)?2:0;
	color += (buf.color&0x00ff0000)?4:0;
	data2 = ((led_value-1)<<24)|(color<<16)|(buf.flashMode<<8)|(buf.brightnessMode);
	printk("proc_comm data2 : 0x%08x\n",data2);
	msm_proc_comm(PCOM_CUSTOMER_CMD2, &data1, &data2);

	data1 = PROC_COMM_SUB_CMD_LED_CONTROL_2;
	data2 = (buf.flashOnMS<<16)|(buf.flashOffMS);
	printk("proc_comm data2 : 0x%08x\n",data2);
	msm_proc_comm(PCOM_CUSTOMER_CMD2, &data1, &data2);

	if(fs_state)queue_work(led_work, &work_led);

	return;

}

static void led_work_0(struct work_struct *work)
{

	struct light_state_t  buf;
	int fd;

	if(!fs_state){
		if((fd = sys_open("/dev/ledstate", O_RDWR|O_CREAT, 0666)) != -1) {
			if (sys_ftruncate(fd, 256) != -1) {
				sys_write(fd, (void*)&buf, sizeof(struct light_state_t));
				sys_close(fd);
				fs_state = 1;
				return;
			}
		}
		mod_timer(&timer_led,jiffies + msecs_to_jiffies(10));
		return;
	}

}

static void led_timer_start(unsigned long data)
{
	queue_work(led_work, &work_led);
}

static struct led_classdev msm_blue_led = {
	.name			= "blue",
	.brightness_set		= msm_blue_led_set,
	.brightness		= LED_OFF,
};

static struct led_classdev msm_state_led = {
	.name			= "state",
	.brightness_set		= msm_state_led_set,
	.brightness		= LED_OFF,
};


static int msm_pmic_led_probe(struct platform_device *pdev)
{
	int rc;

	rc = led_classdev_register(&pdev->dev, &msm_kp_bl_led);
	if (rc) {
		dev_err(&pdev->dev, "unable to register led class driver\n");
		return rc;
	}








	rc = led_classdev_register(&pdev->dev, &msm_button_bl_led);
	if (rc) {
		dev_err(&pdev->dev, "unable to register led class driver\n");
		return rc;
	}
	rc = led_classdev_register(&pdev->dev, &msm_red_led);
	if (rc) {
		dev_err(&pdev->dev, "unable to register led class driver\n");
		return rc;
	}
	rc = led_classdev_register(&pdev->dev, &msm_green_led);
	if (rc) {
		dev_err(&pdev->dev, "unable to register led class driver\n");
		return rc;
	}
	rc = led_classdev_register(&pdev->dev, &msm_blue_led);
	if (rc) {
		dev_err(&pdev->dev, "unable to register led class driver\n");
		return rc;
	}

	rc = led_classdev_register(&pdev->dev, &msm_state_led);
	if (rc) {
		dev_err(&pdev->dev, "unable to register led class driver\n");
		return rc;
	}


	msm_keypad_bl_led_set(&msm_kp_bl_led, LED_OFF);
	msm_button_bl_led_set(&msm_button_bl_led, LED_OFF);


	INIT_WORK(&work_led, led_work_0);
	INIT_WORK(&work_ledstate, ledstate_work_0);
	setup_timer(&timer_led, led_timer_start,0);

	mod_timer(&timer_led,jiffies + msecs_to_jiffies(1000));


	return rc;
}

static int __devexit msm_pmic_led_remove(struct platform_device *pdev)
{
	led_classdev_unregister(&msm_kp_bl_led);
	led_classdev_unregister(&msm_button_bl_led);  


	led_classdev_unregister(&msm_red_led);
	led_classdev_unregister(&msm_green_led);
	led_classdev_unregister(&msm_blue_led);
	led_classdev_unregister(&msm_state_led);


	return 0;
}

#ifdef CONFIG_PM
static int msm_pmic_led_suspend(struct platform_device *dev,
		pm_message_t state)
{
	printk("msm_pmic_led_suspend button:%d, key:%d\n", button_bl_val, key_bl_val);
	led_classdev_suspend(&msm_button_bl_led);  
	led_classdev_suspend(&msm_kp_bl_led);

	return 0;
}

static int msm_pmic_led_resume(struct platform_device *dev)
{
	printk("msm_pmic_led_resume\n");
	led_classdev_resume(&msm_kp_bl_led);
	led_classdev_resume(&msm_button_bl_led);  

	return 0;
}
#else
#define msm_pmic_led_suspend NULL
#define msm_pmic_led_resume NULL
#endif

static struct platform_driver msm_pmic_led_driver = {
	.probe		= msm_pmic_led_probe,
	.remove		= __devexit_p(msm_pmic_led_remove),
	.suspend	= msm_pmic_led_suspend,
	.resume		= msm_pmic_led_resume,
	.driver		= {
		.name	= "pmic-leds",
		.owner	= THIS_MODULE,
	},
};

static int __init msm_pmic_led_init(void)
{

	fs_state = 0;
	led_work = create_singlethread_workqueue("led_work");
	ledstate_work = create_singlethread_workqueue("ledstate_work");

	return platform_driver_register(&msm_pmic_led_driver);
}
module_init(msm_pmic_led_init);

static void __exit msm_pmic_led_exit(void)
{

	if (led_work)
		destroy_workqueue(led_work);

	platform_driver_unregister(&msm_pmic_led_driver);
}
module_exit(msm_pmic_led_exit);

MODULE_DESCRIPTION("MSM PMIC LEDs driver");
MODULE_LICENSE("GPL v2");
MODULE_ALIAS("platform:pmic-leds");
