/*
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2013 KYOCERA Corporation
 */

#include "msm_fb.h"
#include "mddihost.h"
#include "mddihosti.h"

#include <linux/gpio.h>
#include <mach/vreg.h>
#include <mach/hs_io_ctl_a.h>

#include "mddi_oem_ILI9331.h"

#include <mach/kcj_dev_info.h>
#include <linux/wakelock.h>

struct wake_lock lcd_wlock;

#define DEV_LCD_WLOCK_INIT() wake_lock_init(&lcd_wlock, WAKE_LOCK_IDLE   , "lcd_wlock")
void oem_dev_lcd_wakelock(   int type );
void oem_dev_lcd_wakeunlock( int type );
static void lcd_wakelock_core( int onoff );

/* Macro */
#define LCD_CtrlWrite_ILI9331(addr, data)  mddi_queue_register_write(addr, data, TRUE, 0)


static int g_device_lcd_state = TRUE;

/* Function */
static int ili9331_lcd_on(struct platform_device *pdev);
static int ili9331_lcd_off(struct platform_device *pdev);

static struct vreg *vreg_gp3;
void ili9331_vreg_on( void )
{
  int rc = 0;
  vreg_gp3      = vreg_get(NULL, "gp3");
  rc = vreg_set_level(vreg_gp3, 2600);
  if (!rc)
  {
    rc = vreg_enable(vreg_gp3);
  }
}
void ili9331_vreg_off( void )
{
  int rc = 0;
  vreg_gp3      = vreg_get(NULL, "gp3");
  rc = vreg_disable(vreg_gp3);

}
void ili9331_vreg_init( void )
{
  static boolean first_flg = TRUE;
  if(first_flg == TRUE)
  {
    ili9331_vreg_on();
    first_flg = FALSE;
  }
}
/*
    power on and reset function
*/
static int ili9331_lcd_reset( void )
{
    /* reset ili9331 */
    gpio_direction_output(GPIO_ILI9331_RESET,1);
    msleep(2);
    gpio_direction_output(GPIO_ILI9331_RESET,0);
    msleep(15);
    gpio_direction_output(GPIO_ILI9331_RESET,1);
    msleep(75);
    printk(KERN_ERR "%s() called End\n", __func__);
    return(0);
}

/*
    ILI9331 BackLight Enable
*/
void ili9331_backlight_enable(void)
{
    /* LCD BackLight Enable */
    gpio_direction_output(GPIO_LCD_BACKLIGHT_EN,1);
    LCD_CtrlWrite_ILI9331(0x00B3, 0x002C);   /*  */
    LCD_CtrlWrite_ILI9331(0x00C8, 0x0015);   /*  */
    LCD_CtrlWrite_ILI9331(0x00B5, 0x0000);   /*  */
}

void mddi_oem_lcdbl_PWM( unsigned char pwm )
{
  oem_dev_lcd_wakelock(0x02);
  LCD_CtrlWrite_ILI9331(0x00B1, pwm);   /*  */
  oem_dev_lcd_wakeunlock(0x02);
}

void ili9331_backlight_disable(void)
{
  LCD_CtrlWrite_ILI9331(0x00B3, 0x0000);   /*  */
  gpio_direction_output(GPIO_LCD_BACKLIGHT_EN,0);
  msleep(200);
}

/*
    ILI9331 register setting
*/
static int ili9331_resister_init_sequence(void)
{
    if(mddi_oem_lcd_connect_check() == FALSE)
    {
      printk(KERN_ERR "%s() Not LCD Connect No init!! \n", __func__);
      return(0);
    }
    if(g_device_lcd_state == TRUE)
    {
      printk(KERN_ERR "Status == LCD Power ON!!! \n");
      return(0);
    }

  /* LCD Reset */
    ili9331_lcd_reset();

  //************* Start Initial Sequence **********//
    LCD_CtrlWrite_ILI9331(0x00E7, 0x1014);   // Set internal timing
    LCD_CtrlWrite_ILI9331(0x0001, 0x0100);	   // set SS and SM bit
    LCD_CtrlWrite_ILI9331(0x0002, 0x0200);	   // set 1 line inversion
    LCD_CtrlWrite_ILI9331(0x0003, 0xD030);	   // set GRAM write direction and BGR=1. 
    LCD_CtrlWrite_ILI9331(0x0008, 0x0202);	   // set the back porch and front porch
    LCD_CtrlWrite_ILI9331(0x0009, 0x0000);	   // set non-display area refresh cycle ISC[3:0]
    LCD_CtrlWrite_ILI9331(0x000A, 0x0008);	   // FMARK function
    LCD_CtrlWrite_ILI9331(0x000C, 0x0000);    // RGB interface setting
    LCD_CtrlWrite_ILI9331(0x000D, 0x00A0);	   // Frame marker Position MYTEST
    LCD_CtrlWrite_ILI9331(0x000F, 0x0000);     // RGB interface polarity
  //*************Power On sequence ****************//
    LCD_CtrlWrite_ILI9331(0x0010, 0x0000);      // SAP, BT[3:0], AP, DSTB, SLP, STB
    LCD_CtrlWrite_ILI9331(0x0011, 0x0007);      // DC1[2:0], DC0[2:0], VC[2:0]
    LCD_CtrlWrite_ILI9331(0x0012, 0x0000);      // VREG1OUT voltage
    LCD_CtrlWrite_ILI9331(0x0013, 0x0000);      // VDV[4:0] for VCOM amplitude
    msleep(100);                            // Dis-charge capacitor power voltage
    LCD_CtrlWrite_ILI9331(0x0010, 0x1690);      // SAP, BT[3:0], AP, DSTB, SLP, STB
    LCD_CtrlWrite_ILI9331(0x0011, 0x0227);      // Set DC1[2:0], DC0[2:0], VC[2:0]
    msleep(30);			                    // Delay 30ms  
    LCD_CtrlWrite_ILI9331(0x0012, 0x008A);      // External reference voltage= Vci;
    msleep(30);			                    // Delay 30ms
    LCD_CtrlWrite_ILI9331(0x0013, 0x1900);      // VDV[4:0] for VCOM amplitude
    LCD_CtrlWrite_ILI9331(0x0029, 0x002b);      // VCM[5:0] for VCOMH
    LCD_CtrlWrite_ILI9331(0x002B, 0x0009);      // Set Frame Rate(60Hz)
    msleep(30);                              // Delay 30ms
    LCD_CtrlWrite_ILI9331(0x0020, 0x0000);      // GRAM horizontal Address
    LCD_CtrlWrite_ILI9331(0x0021, 0x0000);      // GRAM Vertical Address 
  // ----------- Adjust the Gamma  Curve ----------//
  /* gamma2.4 */
    LCD_CtrlWrite_ILI9331(0x0030, 0x0000);
    LCD_CtrlWrite_ILI9331(0x0031, 0x0405);
    LCD_CtrlWrite_ILI9331(0x0032, 0x0203);
    LCD_CtrlWrite_ILI9331(0x0035, 0x0102);
    LCD_CtrlWrite_ILI9331(0x0036, 0x0004);
    LCD_CtrlWrite_ILI9331(0x0037, 0x0405);
    LCD_CtrlWrite_ILI9331(0x0038, 0x0203);
    LCD_CtrlWrite_ILI9331(0x0039, 0x0707);
    LCD_CtrlWrite_ILI9331(0x003C, 0x0201);
    LCD_CtrlWrite_ILI9331(0x003D, 0x0004);
  //------------------ Set GRAM area ---------------//
    LCD_CtrlWrite_ILI9331(0x0050, 0x0000);	    // Horizontal GRAM Start Address
    LCD_CtrlWrite_ILI9331(0x0051, 0x00EF);	    // Horizontal GRAM End Address
    LCD_CtrlWrite_ILI9331(0x0052, 0x0000);	    // Vertical GRAM Start Address
    LCD_CtrlWrite_ILI9331(0x0053, 0x013F);	    // Vertical GRAM Start Address
    LCD_CtrlWrite_ILI9331(0x0060, 0x2700);	    // Gate Scan Line
    LCD_CtrlWrite_ILI9331(0x0061, 0x0001);	    // NDL,VLE, REV
    LCD_CtrlWrite_ILI9331(0x006A, 0x0000);	    // set scrolling line
  //-------------- Partial Display Control ---------//
    LCD_CtrlWrite_ILI9331(0x0080, 0x0000);
    LCD_CtrlWrite_ILI9331(0x0081, 0x0000);
    LCD_CtrlWrite_ILI9331(0x0082, 0x0000);
    LCD_CtrlWrite_ILI9331(0x0083, 0x0000);
    LCD_CtrlWrite_ILI9331(0x0084, 0x0000);
    LCD_CtrlWrite_ILI9331(0x0085, 0x0000);
  //-------------- Panel Control -------------------//
    LCD_CtrlWrite_ILI9331(0x0090, 0x0010);
    LCD_CtrlWrite_ILI9331(0x0092, 0x0600);
   
    LCD_CtrlWrite_ILI9331(0x0007, 0x0133);	    // 262K color and display ON
/*-------------------------------------------------------------*/
    return(0);
}

static int ili9331_lcd_on(struct platform_device *pdev)
{
    struct msm_fb_data_type *mfd;    
    int ret;

    mfd = platform_get_drvdata(pdev);

    printk(KERN_ERR "%s() Start panel.id = %d img(%x)\n", __func__, mfd->panel.id, mfd->fb_imgType);

    if (!mfd){
      printk(KERN_ERR "%s() error msd", __func__);
        return -ENODEV;
    }
    if (mfd->key != MFD_KEY){
      printk(KERN_ERR "%s() error msd->key = %d\n", __func__, mfd->key);
        return -EINVAL;
    }

    oem_dev_lcd_wakelock(0x01);

    /* LCD POWER ON */
    ili9331_vreg_on();

    ret = ili9331_resister_init_sequence();
    if(ret){
        printk(KERN_ERR "%s():ili9331_resister_init_sequence() Failed!!!!!!!\n", __func__);
    }
    mddi_host_write_pix_attr_reg(MDDI_DEFAULT_PRIM_PIX_ATTR);

    g_device_lcd_state = TRUE;

    printk(KERN_ERR "%s() End %x\n", __func__, ret);
    g5126bl_send_intensity(mddi_oem_lcdbl_value);

    kcj_dev_info_update_lcd_onoff(true);

    oem_dev_lcd_wakeunlock(0x01);
    return 0;
}

static int ili9331_lcd_off(struct platform_device *pdev)
{
    if(mddi_oem_lcd_connect_check() == FALSE)
    {
      printk(KERN_ERR "%s() Not LCD Connect No init!! \n", __func__);
      return(0);
    }
    if(g_device_lcd_state == FALSE)
    {
      printk(KERN_ERR "Status == LCD Power OFF!!! \n");
      return(0);
    }


  printk(KERN_ERR "%s() Start \n", __func__);
  oem_dev_lcd_wakelock(0x01);/* Sleep禁止 *//* F15-SWD0128-00031 */
  /* LCD BackLight OFF */
  g_device_lcd_state = FALSE;
  mddi_oem_lcdbl_PWM(0);
  ili9331_backlight_disable();

  LCD_CtrlWrite_ILI9331(0x0007, 0x0131);
  msleep(10);                            
  LCD_CtrlWrite_ILI9331(0x0007, 0x0130);
  msleep(10);                             
  LCD_CtrlWrite_ILI9331(0x0007, 0x0000);
  
  LCD_CtrlWrite_ILI9331(0x0010, 0x0080);
  LCD_CtrlWrite_ILI9331(0x0011, 0x0000);
  LCD_CtrlWrite_ILI9331(0x0012, 0x0000);
  LCD_CtrlWrite_ILI9331(0x0013, 0x0000);
  msleep(200);
  LCD_CtrlWrite_ILI9331(0x0010, 0x0082);

  gpio_direction_output(GPIO_ILI9331_RESET,0);
  msleep(50);
  ili9331_vreg_off();

    oem_dev_lcd_wakeunlock(0x01);

    printk(KERN_ERR "%s() End \n", __func__);


    kcj_dev_info_update_lcd_onoff(false);

  return 0;
}


/* LCD Power OFF */
void oem_dev_lcd_power_off( void )
{
  ili9331_lcd_off(NULL);
}


static int ili9331_probe(struct platform_device *pdev)
{
printk(KERN_ERR "%s() pdev->name=[%s]\n", __func__, pdev->name);
	msm_fb_add_device(pdev);
	return 0;
}

static struct platform_driver this_driver = {
	.probe  = ili9331_probe,
	.driver = {
		.name   = "mddi_ili9331_vga",
	},
};

static struct msm_fb_panel_data ili9331_panel_data = {
	.on = ili9331_lcd_on,
	.off = ili9331_lcd_off,

	.set_vsync_notifier = NULL,

};

static struct platform_device this_device = {
	.name   = "mddi_ili9331_vga",
	.id	= 0,
	.dev	= {
		.platform_data = &ili9331_panel_data,
	}
};

static int __init ili9331_init(void)
{
	int ret;
	struct msm_panel_info *pinfo;

    printk(KERN_ERR "%s() \n", __func__);

	ret = platform_driver_register(&this_driver);
	if (!ret) {
		pinfo = &ili9331_panel_data.panel_info;
		pinfo->xres     = 240;
		pinfo->yres     = 320;
		pinfo->type     = MDDI_PANEL;
		pinfo->pdest    = DISPLAY_1;
		pinfo->mddi.vdopkt  = MDDI_DEFAULT_PRIM_PIX_ATTR;
		pinfo->wait_cycle   = 0;
		pinfo->bpp      = 18;
		pinfo->fb_num   = 2;
		pinfo->bl_max   = 0xFF;
		pinfo->bl_min   = 0x01;

		pinfo->clk_rate =  61440000;
		pinfo->clk_min  =  61440000;
		pinfo->clk_max  =  61440000;




		pinfo->lcd.vsync_enable     = TRUE;
        /*
        *   refresh cycle is 60Hz
        */
		pinfo->lcd.refx100          = 6018;
		pinfo->lcd.v_back_porch     = 14;
		pinfo->lcd.v_front_porch    = 2;
		pinfo->lcd.v_pulse_width    = 0;
		pinfo->lcd.hw_vsync_mode    = TRUE;
		pinfo->lcd.vsync_notifier_period    = 0;

		ret = platform_device_register(&this_device);
		if (ret)
			platform_driver_unregister(&this_driver);
	}


    gpio_tlmm_config(GPIO_CFG( GPIO_ILI9331_RESET, 0, GPIO_CFG_OUTPUT, GPIO_CFG_PULL_UP, GPIO_CFG_2MA), GPIO_CFG_ENABLE);


  DEV_LCD_WLOCK_INIT();
  lcd_wakelock_core( FALSE );

	return ret;
}

module_init(ili9331_init);





#define CHECK_DATA1 0x9331 
boolean hs_lcd_testmode_frex_check_drv( void )
{
  int     rc;
  uint32  read_data = 0;
  boolean ret = TRUE;

  printk(KERN_ERR "%s:start \n", __func__);


  rc = mddi_queue_register_read( 0x00, &read_data, TRUE, 0);
  if(rc < 0)
  {

    ret = FALSE;
  }
  else
  {
    if(read_data != CHECK_DATA1)
    {
      printk(KERN_ERR "NG!!! read[%x]==check[%x] \n", read_data, CHECK_DATA1);
      ret = FALSE;
    }
  }
  printk(KERN_ERR "frex_check_drv %d:end \n", ret);
  return ret ;
}


boolean mddi_lcd_status_get( void )
{
  return((boolean)g_device_lcd_state);
}

static boolean oem_lcd_connect_status = FALSE;


static boolean _oem_lcd_connect_check_proc( void )
{
  if(HS_GET_LCD_CONNECT())
  {

    return(TRUE);
  }
  else
  {

    return(FALSE);
  }
}





boolean mddi_oem_lcd_connect_check( void )
{
#if defined(CONFIG_TARGET_PROTOTYPE_WS0)

  return(TRUE);
#else
  static boolean first_flg = TRUE;
  if(first_flg == TRUE)
  {
    oem_lcd_connect_status = _oem_lcd_connect_check_proc();
    gpio_tlmm_config(GPIO_CFG( GPIO_LCD_CONNECT, 0, GPIO_CFG_INPUT, GPIO_CFG_NO_PULL, GPIO_CFG_2MA), GPIO_CFG_ENABLE);/* input no-pull */
    printk(KERN_ERR "%s() LCD Connect[%x]GPIO[%x]\n", __func__, oem_lcd_connect_status, GPIO_LCD_CONNECT);
    first_flg = FALSE;
  }
  return(oem_lcd_connect_status);
#endif
}
/*------------------------------------------------------------------*/



void oem_dev_lcd_start_address_set( void )
{
  LCD_CtrlWrite_ILI9331(0x0020, 0x0000);      /* GRAM horizontal Address */
  LCD_CtrlWrite_ILI9331(0x0021, 0x0000);      /* GRAM Vertical   Address */
}




static void lcd_wakelock_core( int onoff )
{
  static int lcd_wlock_flg = -1;

  if(onoff)
  {
    if(lcd_wlock_flg != TRUE)
    {
      wake_lock(&lcd_wlock);
      lcd_wlock_flg = TRUE;
    }
  }

  else
  {
    if(lcd_wlock_flg != FALSE)
    {
      wake_unlock(&lcd_wlock);
      lcd_wlock_flg = FALSE;
    }
  }
}
int lcd_wlock_ctrl_flg = 0;

void oem_dev_lcd_wakelock( int type )
{
  lcd_wlock_ctrl_flg |= type;
  if(lcd_wlock_ctrl_flg)
  {
    lcd_wakelock_core(TRUE);
  }
}

void oem_dev_lcd_wakeunlock( int type )
{
  lcd_wlock_ctrl_flg &= ~type;
  if(lcd_wlock_ctrl_flg == 0)
  {
    lcd_wakelock_core(FALSE);
  }
}


